import sys
import os
import pathlib

import pylogconf.core
from pytconf import register_endpoint, register_main, config_arg_parse_and_launch

from pydmt.features.documentation import Documentation
from pydmt.features.templating import Templating
from pydmt.features.yaml import Yaml
from pydmt.core.pydmt import PyDMT
from pydmt.static import APP_NAME, VERSION_STR, DESCRIPTION


def add_to_path():
    """
    This adds to PYTHONPATH various paths we need
    If you disable this then templates would not be able to find things like 'config/python.py'
    """
    add_import_of_cwd = True
    add_import_of_home = True
    add_import_of_shared = True
    if add_import_of_shared:
        folder = "/etc/pydmt"
        if folder not in sys.path:
            sys.path.insert(0, folder)
    if add_import_of_home:
        folder = os.path.join(str(pathlib.Path.home()), ".config/pydmt")
        if folder not in sys.path:
            sys.path.insert(0, folder)
    if add_import_of_cwd:
        if "" not in sys.path:
            sys.path.insert(0, "")


@register_endpoint(
    description="Build the project",
)
def build():
    add_to_path()

    pylogconf.core.setup()
    p = PyDMT()

    f = Templating()
    f.setup(p)
    f = Documentation()
    f.setup(p)
    f = Yaml()
    f.setup(p)

    stats = p.build_all()
    sys.exit(stats.get_os_error_code())


@register_endpoint(
    description="Clean all generated files"
)
def clean() -> None:
    pass


@register_main(
    app_name=APP_NAME,
    version=VERSION_STR,
    main_description=DESCRIPTION,
)
def main():
    pylogconf.core.setup()
    config_arg_parse_and_launch()


if __name__ == '__main__':
    main()



import os
from typing import Callable, List, Dict


def make_console_script(package_name: str, main: Callable):
    """
    return a string suitable to be used as the section 'console_scripts'
    of the 'entry_points' parameters of setup.py.

    example: 'pycmdtools=pycmdtools.endpoints.main:main'
    :param package_name:
    :param main:
    :return:
    """
    return "{}={}:{}".format(
        package_name,
        getattr(main, '__module__'),
        main.__name__,
    )


def array_indented(level: int, array: List[str], quote_char='\'', comma_after=False) -> str:
    """
    return an array indented according to indent level
    :param level:
    :param array:
    :param quote_char:
    :param comma_after:
    :return:
    """
    out = "[\n"
    for x in array:
        out += (((level + 1) * 4) * " ") + '{qc}{x}{qc}'.format(qc=quote_char, x=x) + ",\n"
    out += ((level * 4) * " ") + "]"
    if comma_after:
        out += ","
    return out


def dict_indented(level: int, dictionary: Dict[str, List[int]], quote_char='\'', comma_after=False) -> str:
    """
    return an dict indented according to indent level
    :param level:
    :param dictionary:
    :param quote_char:
    :param comma_after:
    :return:
    """
    out = "{\n"
    for k, v in dictionary.items():
        spaces = (((level + 1) * 4) * " ")
        out += '{spaces}{qc}{k}{qc}: {v},\n'.format(
            spaces=spaces,
            qc=quote_char,
            k=k,
            v=v,
        )
    spaces = ((level * 4) * " ")
    out += "{}}}".format(spaces)
    if comma_after:
        out += ","
    return out


def find_packages(path: str) -> List[str]:
    """
    A better version of find_packages than what setuptools offers
    This function needs to be deterministic.
    :param path:
    :return:
    """
    ret = []
    for root, _dir, files in os.walk(path):
        if '__init__.py' in files:
            ret.append(root.replace("/", "."))
    return sorted(ret)



import os

from pydmt.api.feature import Feature
from pydmt.builders.sphinx import Sphinx
from pydmt.core.pydmt import PyDMT


class Documentation(Feature):
    def __init__(
        self,
        sphinx_folder: str = "sphinx",
    ):
        self.sphinx_folder = sphinx_folder
        # pylint: disable=import-outside-toplevel
        try:
            import config.project
            self.project_name = config.project.project_name
        except ModuleNotFoundError:
            pass

    def setup(self, pydmt: PyDMT) -> None:
        if not os.path.isdir(self.sphinx_folder):
            return
        pydmt.add_builder(Sphinx(package_name=self.project_name))



import yaml

from pydmt.utils.filesystem import mkdir_touch

from pydmt.builders.one_source_one_target import OneSourceOneTarget


class YamlValidate(OneSourceOneTarget):
    def build(self):
        with open(self.source, "rt") as input_handle:
            yaml.load(input_handle, yaml.SafeLoader)
        mkdir_touch(self.target)



def to_php(x):
    if isinstance(x, str):
        return '\'{0}\''.format(x)
    if isinstance(x, bool):
        if x:
            return 'TRUE'
        return 'FALSE'
    if x is None:
        return 'null'
    raise ValueError('dont know how to translate type', type(x), x)



import os
import glob
import pprint


def hlp_source_under(folder):
    """
    this function finds all the python packages under a folder and
    write the 'packages' and 'package_dir' entries for a python setup.py
    script
    """
    # walk the folder and find the __init__.py entries for packages.
    packages = []
    package_dir = {}
    for root, _, files in os.walk(folder):
        for file in files:
            if file != '__init__.py':
                continue
            full = os.path.dirname(os.path.join(root, file))
            relative = os.path.relpath(full, folder)
            packages.append(relative)
            package_dir[relative] = full
    # we use pprint because we want the order to always remain the same
    return 'packages={0},\npackage_dir={1}'.format(sorted(packages), pprint.pformat(package_dir))


def hlp_files_under(destination_folder, pat):
    return '(\'{0}\', {1})'.format(destination_folder, [x for x in glob.glob(pat) if os.path.isfile(x)])


def make_hlp_project_keywords(d):
    def hlp_project_keywords():
        return '{0}'.format(d.project_keywords.split())

    return hlp_project_keywords


def make_hlp_project_platforms(d):
    def hlp_project_platforms():
        return '{0}'.format(d.project_platforms.split())

    return hlp_project_platforms


def make_hlp_project_classifiers(d):
    def hlp_project_classifiers():
        classifiers = d.project_classifiers.split('\n')
        classifiers = [x.strip()[1:-1] for x in classifiers]
        return '{0}'.format(classifiers)

    return hlp_project_classifiers


def make_hlp_wrap(level):
    def hlp_wrap(t):
        return t.replace('\n', '\n' + '\t' * level)

    return hlp_wrap



import abc
from typing import List, Generator, Tuple

from pydmt.utils.digester import Digester


class Node(abc.ABC):
    """
    This is a source object which knows how to add his checksum to a checksum
    calculation
    """
    @abc.abstractmethod
    def add_to_digester(self, d: Digester) -> None:
        pass

    @abc.abstractmethod
    def get_name(self) -> str:
        pass


class Builder(abc.ABC):
    """
    A Builder is what really builds things in the system.
    A builder knows on which inputs it relies and what
    outputs it generates (sometimes before build and sometimes
    after).
    """

    def get_name(self) -> str:
        """
        Override this to get better names
        :return: the name of this builder
        """
        return f"{self.__class__.__name__}"

    @abc.abstractmethod
    def __init__(self):
        """ initialize your builder here """

    @abc.abstractmethod
    def build(self) -> None:
        """
        this method actually does the building
        Just do whatever you want here. Options are:
        - Write pure python code
        - Call native code
        - Call external programs
        - A combination of the above
        If there are any problems then throw an exception.
        Try not to segfault the interpreter in this method...:)
        """

    @abc.abstractmethod
    def get_sources(self) -> List[Node]:
        """
        return the name of the source files for this builder
        If the builder takes a whole folder the list all the filers in that folder.
        If a built takes all the .py files in a folder then list those.
        In the current implementation this method is not really that important because
        it is not used to calculate the signature of the input to the build.
        The @get_signature method is use for that.
        In the future the get_signature method will go away.
        """
    @abc.abstractmethod
    def get_targets(self) -> List[Node]:
        """
        return list of targets
        """

    @abc.abstractmethod
    def yield_results(self) -> Generator[Tuple[str, str], None, None]:
        """
        Return the signatures and names of results
        :return:
        """

    def get_signature(self) -> str:
        """
        return the sha1 of anything that identifies the sources of the build
        Techically this is the sha1 of the file content of the list of files
        returned from :func:`~get_sources`
        """
        d = Digester()
        for source in self.get_sources():
            source.add_to_digester(d)
        return d.get_hexdigest()

    def get_targets_as_string(self) -> str:
        return ",".join([x.get_name() for x in self.get_targets()])


class File(Node):
    """
    This is a node which is a file
    """
    def __init__(self, filename: str):
        self.filename = filename

    def add_to_digester(self, d: Digester) -> None:
        d.add_file(self.filename)

    def get_name(self):
        return self.filename


class SourceFile(File):
    pass


class TargetFile(File):
    pass


class SourceFiles(Node):
    """
    This is a source of many files
    """
    def __init__(self, filenames: List[str], name: str):
        self.filenames = filenames
        self.name = name

    def add_to_digester(self, d: Digester) -> None:
        d.add_files(self.filenames)

    def get_name(self):
        return self.name


class Folder(Node):
    """
    This is a node representing a single Folder
    """
    def __init__(self, folder: str):
        self.folder = folder

    def add_to_digester(self, d: Digester) -> None:
        d.add_folders(folders=[self.folder])

    def get_name(self):
        return self.folder


class SourceFolder(Folder):
    pass


class TargetFolder(Folder):
    pass
